# Generated by roxygen2: do not edit by hand

S3method(Biclustering,binary)
S3method(Biclustering,default)
S3method(Biclustering,nominal)
S3method(Biclustering,ordinal)
S3method(Biclustering,rated)
S3method(Biclustering_IRM,binary)
S3method(Biclustering_IRM,default)
S3method(Biclustering_IRM,nominal)
S3method(Biclustering_IRM,ordinal)
S3method(Biclustering_IRM,rated)
S3method(CCRR,binary)
S3method(CCRR,default)
S3method(CCRR,nominal)
S3method(Dimensionality,binary)
S3method(Dimensionality,default)
S3method(Dimensionality,ordinal)
S3method(Dimensionality,rated)
S3method(DistractorAnalysis,LRArated)
S3method(DistractorAnalysis,ratedBiclustering)
S3method(ITBiserial,binary)
S3method(ITBiserial,default)
S3method(InterItemAnalysis,binary)
S3method(InterItemAnalysis,default)
S3method(InterItemAnalysis,ordinal)
S3method(ItemEntropy,binary)
S3method(ItemEntropy,default)
S3method(ItemEntropy,ordinal)
S3method(ItemLift,binary)
S3method(ItemLift,default)
S3method(ItemOdds,binary)
S3method(ItemOdds,default)
S3method(ItemStatistics,binary)
S3method(ItemStatistics,default)
S3method(ItemStatistics,ordinal)
S3method(ItemThreshold,binary)
S3method(ItemThreshold,default)
S3method(ItemThreshold,ordinal)
S3method(ItemTotalCorr,binary)
S3method(ItemTotalCorr,default)
S3method(ItemTotalCorr,ordinal)
S3method(JCRR,binary)
S3method(JCRR,default)
S3method(JCRR,nominal)
S3method(JointSampleSize,binary)
S3method(JointSampleSize,default)
S3method(LCA,binary)
S3method(LCA,default)
S3method(LCA,nominal)
S3method(LCA,rated)
S3method(LRA,binary)
S3method(LRA,default)
S3method(LRA,ordinal)
S3method(LRA,rated)
S3method(M2,LRAordinal)
S3method(M2,default)
S3method(M2,nominalBiclustering)
S3method(M2,nominalLCA)
S3method(M2,ordinalBiclustering)
S3method(M2,ratedLCA)
S3method(MutualInformation,binary)
S3method(MutualInformation,default)
S3method(MutualInformation,ordinal)
S3method(PhiCoefficient,binary)
S3method(PhiCoefficient,default)
S3method(PolychoricCorrelationMatrix,default)
S3method(PolychoricCorrelationMatrix,ordinal)
S3method(TestStatistics,binary)
S3method(TestStatistics,default)
S3method(TestStatistics,ordinal)
S3method(TetrachoricCorrelationMatrix,binary)
S3method(TetrachoricCorrelationMatrix,default)
S3method(add_M2,LRAordinal)
S3method(add_M2,default)
S3method(add_M2,nominalBiclustering)
S3method(add_M2,nominalLCA)
S3method(add_M2,ordinalBiclustering)
S3method(add_M2,ratedLCA)
S3method(crr,binary)
S3method(crr,default)
S3method(nrs,binary)
S3method(nrs,default)
S3method(passage,binary)
S3method(passage,default)
S3method(percentile,binary)
S3method(percentile,default)
S3method(plot,DistractorAnalysis)
S3method(plot,exametrika)
S3method(print,DistractorAnalysis)
S3method(print,exametrika)
S3method(sscore,binary)
S3method(sscore,default)
S3method(stanine,binary)
S3method(stanine,default)
export(AlphaCoefficient)
export(BINET)
export(BNM)
export(BNM_GA)
export(BNM_PBIL)
export(Biclustering)
export(Biclustering_IRM)
export(BiserialCorrelation)
export(CCRR)
export(CSR)
export(CTT)
export(Dimensionality)
export(DistractorAnalysis)
export(GRM)
export(Glasso)
export(GridSearch)
export(IIF2PLM)
export(IIF3PLM)
export(IRT)
export(ITBiserial)
export(InterItemAnalysis)
export(ItemEntropy)
export(ItemFit)
export(ItemInformationFunc)
export(ItemLift)
export(ItemOdds)
export(ItemReport)
export(ItemStatistics)
export(ItemThreshold)
export(ItemTotalCorr)
export(JCRR)
export(JSR)
export(JointSampleSize)
export(LCA)
export(LDB)
export(LDLRA)
export(LDLRA_PBIL)
export(LRA)
export(LogisticModel)
export(M2)
export(MutualInformation)
export(OmegaCoefficient)
export(PhiCoefficient)
export(PolychoricCorrelationMatrix)
export(RaschModel)
export(ScoreReport)
export(StudentAnalysis)
export(TestFit)
export(TestFitSaturated)
export(TestInformationFunc)
export(TestResponseFunc)
export(TestStatistics)
export(TetrachoricCorrelationMatrix)
export(ThreePLM)
export(TwoPLM)
export(add_M2)
export(calcFitIndices)
export(chatterjee_matrix)
export(chatterjee_xi)
export(crr)
export(dataFormat)
export(grm_iif)
export(grm_prob)
export(longdataFormat)
export(nrs)
export(passage)
export(percentile)
export(polychoric)
export(polyserial)
export(sscore)
export(stanine)
export(tetrachoric)
export(xi_stable)
importFrom(Rcpp,evalCpp)
importFrom(grDevices,dev.off)
importFrom(grDevices,gray)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,jpeg)
importFrom(grDevices,pdf)
importFrom(grDevices,png)
importFrom(grDevices,rainbow)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,curve)
importFrom(graphics,image)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot.new)
importFrom(graphics,rect)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,as_adjacency_matrix)
importFrom(igraph,as_data_frame)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,layout_on_grid)
importFrom(igraph,layout_with_fr)
importFrom(igraph,plot.igraph)
importFrom(mvtnorm,pmvnorm)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,density)
importFrom(stats,dnorm)
importFrom(stats,ecdf)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,optim)
importFrom(stats,optimize)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(utils,combn)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,tail)
importFrom(utils,write.table)
useDynLib(exametrika, .registration=TRUE)
