#' @title J5S10
#' @description A binary response dataset for test analysis
#' @format An exametrika class object with 5 students and 10 items containing binary (0/1) responses
#' @source \url{http://sh0j1ma.stars.ne.jp/exmk/}
"J5S10"

#' @title J15S500
#' @description A binary response dataset for test analysis
#' @format An exametrika class object with 500 students and 15 items containing binary (0/1) responses
#' @source \url{http://sh0j1ma.stars.ne.jp/exmk/}
"J15S500"

#' @title J20S400
#' @description A binary response dataset for test analysis
#' @format An exametrika class object with 400 students and 20 items containing binary (0/1) responses
#' @source \url{http://sh0j1ma.stars.ne.jp/exmk/}
"J20S400"

#' @title J35S515
#' @description A binary response dataset for test analysis
#' @format An exametrika class object with 515 students and 35 items containing binary (0/1) responses
#' @source \url{http://sh0j1ma.stars.ne.jp/exmk/}
"J35S515"

#' @title J12S5000
#' @description A binary response dataset for test analysis
#' @format An exametrika class object with 5000 students and 12 items containing binary (0/1) responses
#' @source \url{http://sh0j1ma.stars.ne.jp/exmk/}
"J12S5000"

#' @title J15S3810
#' @description An ordinal response dataset for test analysis
#' @format An exametrika class object with 3810 students and 15 items containing ordinal responses with 4 categories
"J15S3810"

#' @title J35S5000
#' @description A rated response dataset for test analysis
#' @format An exametrika class object with 5000 students and 35 items containing polytomous responses with correct answers
"J35S5000"

#' @title J5S1000
#' @description A simulated ordinal dataset for test analysis. This is a synthetic dataset
#'   generated using random number generation for demonstration and testing purposes.
#' @format An exametrika class object with 1000 students and 5 items containing ordinal responses
"J5S1000"

#' @title J50S100
#' @description A simulated binary dataset for test analysis. This is a synthetic dataset
#'   generated using random number generation for demonstration and testing purposes.
#' @format An exametrika class object with 100 students and 50 items containing binary responses
"J50S100"

#' @title J35S500
#' @description A simulated ordinal dataset for polytomous Biclustering analysis.
#'   This is a synthetic dataset generated using random number generation
#'   with a cumulative staircase pattern (5 latent classes, 5 fields,
#'   5 response categories) for demonstration and testing purposes.
#'   Contains approximately 0.5\% missing values.
#' @format An exametrika class object with 500 students and 35 items containing ordinal responses with 5 categories
"J35S500"

#' @title J20S600
#' @description A simulated nominal dataset for polytomous Biclustering analysis.
#'   This is a synthetic dataset generated using random number generation
#'   with a cyclic category preference pattern (5 latent classes, 4 fields,
#'   4 response categories) for demonstration and testing purposes.
#'   Contains approximately 0.5\% missing values.
#' @format An exametrika class object with 600 students and 20 items containing nominal responses with 4 categories
"J20S600"

#' @title J21S300
#' @description A simulated rated dataset for Rated IRM and Biclustering analysis.
#'   This is a synthetic dataset generated with a known structure
#'   (5 latent classes, 3 fields, 4 response categories with correct answers)
#'   for demonstration and testing purposes. Each field contains 7 items.
#' @format An exametrika class object with 300 students and 21 items containing
#'   rated responses (multiple-choice with correct answers) with 4 categories
"J21S300"
