# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

simple_quadrature_grm <- function(n_points = 51L) {
    .Call(`_exametrika_simple_quadrature_grm`, n_points)
}

target_to_params_grm <- function(target, nitems, ncat) {
    .Call(`_exametrika_target_to_params_grm`, target, nitems, ncat)
}

log_lik_grm_cpp <- function(target, dat, n_quad_points = 51L) {
    .Call(`_exametrika_log_lik_grm_cpp`, target, dat, n_quad_points)
}

score_function_analytical_grm <- function(target, dat, n_quad_points = 51L) {
    .Call(`_exametrika_score_function_analytical_grm`, target, dat, n_quad_points)
}

score_function_numerical_grm <- function(target, dat, n_quad_points = 51L) {
    .Call(`_exametrika_score_function_numerical_grm`, target, dat, n_quad_points)
}

compare_gradients_grm <- function(target, dat, n_quad_points = 51L) {
    .Call(`_exametrika_compare_gradients_grm`, target, dat, n_quad_points)
}

irm_gibbs_core_cpp <- function(Uq_vec, Uq_dim, Z, cls01_init, fld01_init, gamma_c, gamma_f, alpha_vec_R, max_iter, stable_limit, verbose) {
    .Call(`_exametrika_irm_gibbs_core_cpp`, Uq_vec, Uq_dim, Z, cls01_init, fld01_init, gamma_c, gamma_f, alpha_vec_R, max_iter, stable_limit, verbose)
}

qBiNormal_cpp <- function(a, b, rho) {
    .Call(`_exametrika_qBiNormal_cpp`, a, b, rho)
}

polychoric_likelihood_cpp <- function(rho, mat) {
    .Call(`_exametrika_polychoric_likelihood_cpp`, rho, mat)
}

polychoric_cpp <- function(x, y) {
    .Call(`_exametrika_polychoric_cpp`, x, y)
}

polychoric_matrix_cpp <- function(data) {
    .Call(`_exametrika_polychoric_matrix_cpp`, data)
}

