% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_dataset_explain.R
\docType{data}
\name{J35S500}
\alias{J35S500}
\title{J35S500}
\format{
An exametrika class object with 500 students and 35 items containing ordinal responses with 5 categories
}
\usage{
J35S500
}
\description{
A simulated ordinal dataset for polytomous Biclustering analysis.
This is a synthetic dataset generated using random number generation
with a cumulative staircase pattern (5 latent classes, 5 fields,
5 response categories) for demonstration and testing purposes.
Contains approximately 0.5\\% missing values.
}
\keyword{datasets}
