% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/07_Biclustering.R
\name{softmax}
\alias{softmax}
\title{softmax function}
\usage{
softmax(x)
}
\arguments{
\item{x}{numeric vector}
}
\description{
to avoid overflow
}
