% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_dataset_explain.R
\docType{data}
\name{J21S300}
\alias{J21S300}
\title{J21S300}
\format{
An exametrika class object with 300 students and 21 items containing
rated responses (multiple-choice with correct answers) with 4 categories
}
\usage{
J21S300
}
\description{
A simulated rated dataset for Rated IRM and Biclustering analysis.
This is a synthetic dataset generated with a known structure
(5 latent classes, 3 fields, 4 response categories with correct answers)
for demonstration and testing purposes. Each field contains 7 items.
}
\keyword{datasets}
