% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_dataset_explain.R
\docType{data}
\name{J20S600}
\alias{J20S600}
\title{J20S600}
\format{
An exametrika class object with 600 students and 20 items containing nominal responses with 4 categories
}
\usage{
J20S600
}
\description{
A simulated nominal dataset for polytomous Biclustering analysis.
This is a synthetic dataset generated using random number generation
with a cyclic category preference pattern (5 latent classes, 4 fields,
4 response categories) for demonstration and testing purposes.
Contains approximately 0.5\\% missing values.
}
\keyword{datasets}
